
# q-expansion of newform 320.4.a.c, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 320
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-6], [5], [34], [16], [-58], [-70], [4], [134], [242], [-100], [438], [-138], [178], [-22], [-162], [-268], [-250], [422], [852], [306], [456], [434], [-726], [1378], [-126], [1262], [510], [-26], [1242], [978], [-912], [-926], [516], [958], [-332], [1022], [-926], [-654], [1294], [-2836], [-1742], [-4460], [-3782], [-4474], [-3608], [-256], [5158], [-2226], [-2086], [-5718], [3624], [-82], [-5040], [-2310], [4110], [-746], [4596], [2206], [8278], [1178], [-106], [8134], [4396], [4826], [-7026], [8808], [5602], [-6634], [-3198], [-5230], [312], [-10790], [4190], [-6980], [-13962], [-3810], [9158], [4866], [13486], [5628], [-7938], [-1916], [-16510], [1256], [-12222], [-5946], [1258], [-16422], [-2658], [3686], [-88], [14714], [-7344], [1604], [-14802], [22514], [-6710], [7930], [-4918], [-3922], [-17786], [20266], [13358], [16360], [-15574], [6654], [-17742], [-15840], [-3002], [23610], [-23850], [-5334], [-2164], [25220], [-12306], [-27414], [21834], [23998], [-32004], [8526], [8178], [16646], [-22446], [35336], [-3482], [19402], [9896], [-494], [-9282], [-3252], [4710], [-25764], [30094], [22362], [-30398], [-1290], [14], [38814], [27402], [-28576], [-31762], [-20506], [13014], [22790], [-23696], [-5306], [-21054], [7364], [-17226], [-21202], [-29490], [2570], [-36334], [-12474], [41132], [38416], [41302], [-26150], [-35254], [18550], [17322], [-35190], [-40696], [44306], [18798], [-2468], [61086], [-23854], [-10778], [31100], [3530], [-60116], [-35470], [-49616], [-7974], [-5528], [-9902], [-31018], [47894], [-7046], [-39552], [5326], [50962], [47398], [16810], [-32138], [16682], [-42898], [-5140], [-9046], [2802], [-9736], [-46874], [61966], [-39574], [-37650], [-8082], [15458], [42430], [31698], [73604], [-3026], [-18798], [86916], [-338], [39008], [74650], [1290], [-47752], [32794], [-16562], [-21362], [-16674], [-20560], [-17562], [-35614], [-3790], [12498], [39830], [88894], [-92800], [-56750], [21102], [71454], [25550], [-23182], [62656], [26698], [6920], [-30538], [103860], [38500], [-27098], [-50358], [-103086], [-16410], [-68370], [-85220], [-15012], [-114686], [-87536], [40598], [-77086], [11714], [-54536], [28994], [93848], [580], [-67298], [35574], [42130], [89370], [56702], [46606], [4020], [26530], [-77850], [-7122], [8682], [94358], [-87394], [-99558], [35926], [-97846], [10708], [-8110], [40410], [113538], [-91258], [-110726], [-37946], [-64086], [86704], [38666], [-11642], [37414], [5890], [-64234], [99552], [-18810], [-21540], [-27734], [35906], [55150], [105788], [-111526], [53142], [29040], [158550], [50966], [25854], [125922], [-97056], [-105106], [87222], [-13652], [90726], [71836], [-167466], [12162], [21438], [84904], [-118870], [44208], [36514], [145494], [-112414], [-163464], [-113546], [-56698], [35946], [22434], [47434], [-28478], [64666], [-8924], [166188], [43930], [-98186], [22664], [-106126], [37734], [-146866], [-36686], [-112002], [14260], [143570], [-30950], [179502], [-39434], [174014], [-108136], [-163838], [181024], [89502], [-106730], [8922], [169990], [100530], [-20906], [-80550], [-94642], [60340], [202550], [-206732], [79262], [-154682], [-85188], [-182874], [-17504], [29274], [63350], [-195506], [-126806], [42754], [5064], [-103304], [-156302], [-141578], [-131858], [98842], [-81542], [-121308], [38862], [-38342], [137958], [120958], [55430], [-201792], [-189220], [43678], [-82146], [852], [-87818], [-33996], [166636], [42386], [59238], [162674], [-107030], [-135230], [203274], [-113454], [-64188], [-53146], [201092], [-169050], [-147678], [110522], [111670], [-211218], [192036], [143062], [-182780], [251714], [-28472], [-68598], [19184], [-276690], [-35754], [-172230], [-150886], [246018], [67326], [163188], [-160594], [82674], [247378], [95004], [-136910], [-82554], [19898], [128080], [70978], [-161670], [-114016], [151970], [209058], [-131706], [214374], [93014], [2666], [-129788], [-88422], [212142], [70698], [-52338], [-192952], [210568]]
